#!python3
# https://nagios-plugins.org/doc/guidelines.html

# Import required libs
from .plugin_check import curlCheck
from .oauth2_token import GetOauth2Token 
from .historian_check import HistorianChecks
import argparse
import sys


# Return codes expected by Nagios
OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3

# Return message
message = {
    'status': OK,
    'summary': 'Example summary',
    'perfdata': 'label1=0;;;; '  # 'label'=value[UOM];[warn];[crit];[min];[max] 
}

# For multiple perdata, ensure to add space after each perfdata
# message['perfdata'] = 'label1=x;;;; '
# message['perfdata'] += 'label2=x;;;; '

# Function to parse arguments
def parse_args(args):
    """
    Information extracted from: https://mkaz.com/2014/07/26/python-argparse-cookbook/
     https://docs.python.org/3/library/argparse.html
    :return: parse.parse_args(args) object
    You can use obj.option, example:
    options = parse_args(args)
    options.user # to read username
    """
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter, 
                                     description='nagios plugin to check some url using curl and some other code')

    parser.add_argument('-u', '--url', dest='url', nargs='?', default=None, const=None,
                        help='url to check \n')
    parser.add_argument('-e', '--extra_args', dest='extra_args', nargs='?', default='', const=None,
                            help='extra args to add to curl, see curl manpage  \n')
    
    # Arguments to check using OAuth2
    parser.add_argument('--client_id', dest='client_id', nargs='?', default=None, const=None,
                            help='oauth2 client_id example client id: user01 \n')
    parser.add_argument('--client_secret', dest='client_secret', nargs='?', default=None, const=None,
                            help='oauth2 client_secret client password \n')
    parser.add_argument('--grant_type', dest='grant_type', nargs='?', default='client_credentials', const=None,
                            help='oauth2 grant_type \n')
    parser.add_argument('--auth_url', dest='auth_url', nargs='?', default=None, const=None,
                            help='oauth2 auth_url example: https://login.microsoftonline.com/company.onmicrosoft.com/oauth2/v2.0/token \n')
    parser.add_argument('--instance', dest='instance', nargs='?', default=None, const=None,
                        help='instance name of historian \n')
    parser.add_argument('--oauth2', action='store_true',
                            help='''Flag to use or not token for oauth2 before creating the request, used to check published services that uses azure oauth2 \n
                                    See https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow#refresh-the-access-token \n''')

    if not args:
        raise SystemExit(parser.print_help())

    return parser.parse_args(args)

# Function to execute cli commands
def cli_execution(options):
    """
    : param: options: arguments from parse.parse_args(args) (see parse_args function)
    """
    auth_args = ''
    # Creating access_token to authenticate with azure oauth2
    if options.oauth2:
        if not options.client_id:
            sys.exit('param client_id is required  when using oauth2')
        if not options.client_secret:
            sys.exit('param client_secret is required  when using oauth2')
        if not options.auth_url:
            sys.exit('param auth_url is required  when using oauth2')
        
        #Define oauth object
        oauth_obj = GetOauth2Token(client_id = options.client_id,                              
                               client_secret = options.client_secret,
                               grant_type = options.grant_type,
                               auth_url = options.auth_url)
        
        #Get tuple with http_code and token
        auth_tuple = oauth_obj.get_token()

        #Vaildate http_code
        auth_http_code = auth_tuple[1]
        if auth_http_code != 200:
            sys.exit('Error getting access_token, http_code != 200, http_code: {}'.format(auth_http_code))
        
        #Extract from tuple the token
        access_token = auth_tuple[0]['access_token']      
        header_token = {"Authorization": "Bearer {}".format(access_token)}
        # https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow#use-the-access-token
        auth_args = "--header 'Authorization: {}'".format(header_token['Authorization'])

    #Create historian object    
    historianobj = HistorianChecks(url=options.url,
                              access_token=access_token,
                              instance=options.instance)

    def collect_data():        
        retrcode, msgdata = historianobj.check_tags_data()
        return retrcode, msgdata
    
    def check(retrcode):
        if retrcode >= 2:
            status = CRITICAL
            message['summary'] = 'CRITICAL: '
        elif retrcode == 1:
            status = WARNING
            message['summary'] = 'WARNING: '
        else:
            status = OK
            message['summary'] = 'OK: '
        return status

    # Check logic starts here
    data = collect_data()
    message['status'] = check(data[0])
    # Add summary
    message['summary'] += data[1]
    # Add perfdata

    # total = len(data)
    #message['perfdata'] = curlnagiosobj.format_perfdata()

    # Print the message
    #print("{summary}|{perfdata}".format(
    #    summary=message.get('summary'),
    #    perfdata=message.get('perfdata')
    #))
    # Exit with status code
    raise SystemExit(message['status'])

# Argument parser
# https://docs.python.org/3.5/library/argparse.html

def main():
    """
    Main function
    """
    # Get options with argparse
    options = parse_args(sys.argv[1:])
    # Execute program functions passing the options collected
    cli_execution(options)


if __name__ == "__main__":
    main()



#
# documentation:
# https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow
import requests
import json

# Will support with access token

class GetOauth2Token:
    def __init__(self, client_id, client_secret, auth_url,
                 grant_type = 'client_credentials'):
        """
        reference: https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow#request-an-access-token
        The Authority configured on the client MUST match the one configured on the server. 
        The Scope configured on the client MUST match the one configured on the server, followed by the .default suffix. 
        The Client Id is the ApplicationId of the client application configured on the Microsoft Application Registration Portal. 
        The Client Key is the Password generated on the client. 

        param client_id: example client id: 6731de76-14a6-49ae-97bc-6eba6914391e
        param scope: example: https://company.onmicrosoft.com/some-unique-number-for-scope/.default
        param client_secret: "client password secret here"
        param grant_type: default is "client_credentials"
        param auth_url: use the correct one for your organization and application server, default example
                        https://login.microsoftonline.com/company.onmicrosoft.com/oauth2/v2.0/token
        """
        self.client_id = client_id
        self.client_secret = client_secret
        self.grant_type = grant_type
        self.auth_url = auth_url
    
    def get_token(self):
        """

        return: tuple dict with access_token, status_code
            {'access_token': 'tokenid'
            'expires_in': 3600,
            'ext_expires_in': 0,
            'token_type': 'Bearer'}, 200
        """
        # Request access token:
        # https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow#request-an-access-token

        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        url = self.auth_url
        data = { "client_id": self.client_id,
                "client_secret": self.client_secret,
                "grant_type": self.grant_type
            }

        # requests doc http://docs.python-requests.org/en/v0.10.7/user/quickstart/#custom-headers
        r = requests.post(url=url, data=data, headers=headers, verify=False)

        return r.json(), r.status_code

    def url_check(self, url_check):
        """
        Just example howto proceed with the token got

        param url_check: url to check with authenticated token

        """
        # Use the access token:
        # https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow#use-the-access-token
        
        get_token = self.get_token()
        access_token = get_token[0]['access_token']
        header_token = {"Authorization": "Bearer {}".format(access_token)}
        rt = requests.get(url=url_check, headers=header_token)

        return rt.json(), rt.status_code

# Refresh? will need to check if we are going to do it (don't think so)
# https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow#refresh-the-access-token



#
# documentation:
import requests
import json
import arrow

# Return codes expected by Nagios
OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3


class HistorianChecks:
    def __init__(self, url, access_token, instance):
        
        #initial
        self.url = url
        self.access_token = access_token
        self.instance = instance

        #Declare vars for tags with instance
        self.STATUS_COM_010 = '{}'.format(self.instance) + '.STATUS_COM_010.F_CV'        
        self.STATUS_COM_073 = '{}'.format(self.instance) + '.STATUS_COM_073.F_CV'
        self.STATUS_COM_075 = '{}'.format(self.instance) + '.STATUS_COM_075.F_CV'
        self.STATUS_COM_079 = '{}'.format(self.instance) + '.STATUS_COM_079.F_CV'
        self.STATUS_COM_080 = '{}'.format(self.instance) + '.STATUS_COM_080.F_CV'
        self.STATUS_COM_110 = '{}'.format(self.instance) + '.STATUS_COM_110.F_CV'
        self.STATUS_COM_120 = '{}'.format(self.instance) + '.STATUS_COM_120.F_CV'
        self.STATUS_COM_130 = '{}'.format(self.instance) + '.STATUS_COM_130.F_CV'
        self.STATUS_COM_141 = '{}'.format(self.instance) + '.STATUS_COM_141.F_CV'
        self.STATUS_COM_142 = '{}'.format(self.instance) + '.STATUS_COM_142.F_CV'
        self.STATUS_COM_144 = '{}'.format(self.instance) + '.STATUS_COM_144.F_CV'
        self.STATUS_COM_148 = '{}'.format(self.instance) + '.STATUS_COM_148.F_CV'
        self.STATUS_COM_151 = '{}'.format(self.instance) + '.STATUS_COM_151.F_CV'
        self.STATUS_COM_152 = '{}'.format(self.instance) + '.STATUS_COM_152.F_CV'
        self.STATUS_COM_155 = '{}'.format(self.instance) + '.STATUS_COM_155.F_CV'
        self.STATUS_COM_160 = '{}'.format(self.instance) + '.STATUS_COM_160.F_CV'
        self.STATUS_COM_192 = '{}'.format(self.instance) + '.STATUS_COM_192.F_CV'
        self.STATUS_COM_193 = '{}'.format(self.instance) + '.STATUS_COM_193.F_CV'
        self.STATUS_SAC_CYCLES_SEC = '{}'.format(self.instance) + '.STATUS_SAC_CYCLES_SEC.F_CV'
        self.STATUS_SAC_OVERRUNS = '{}'.format(self.instance) + '.STATUS_SAC_OVERRUNS.F_CV'
        self.STATUS_SAC_STATUS = '{}'.format(self.instance) + '.STATUS_SAC_STATUS.F_CV'
        self.STATUS_WS_ACTIVE_SESSIONS = '{}'.format(self.instance) + '.STATUS_WS_ACTIVE_SESSIONS.F_CV'
        self.STATUS_WS_CLIENT_CONNECTIONS = '{}'.format(self.instance) + '.STATUS_WS_CLIENT_CONNECTIONS.F_CV'
        self.STATUS_WS_HOST_CONNECTIONS = '{}'.format(self.instance) + '.STATUS_WS_HOST_CONNECTIONS.F_CV'
        self.STATUS_WS_MAXIMUM_SESSIONS = '{}'.format(self.instance) + '.STATUS_WS_MAXIMUM_SESSIONS.F_CV'
        self.STATUS_WS_SERVICE = '{}'.format(self.instance) + '.STATUS_WS_SERVICE.F_CV'
        self.STATUS_LAST_UPDATE = '{}'.format(self.instance) + '.STATUS_LAST_UPDATE.A_CV'

    def get_tags_data(self):
    

        # Request JSON machines
        #headers = {'accept': 'application/json'}
        header_token = {"Authorization": "Bearer {}".format(self.access_token)}

        #Add tags to the URL
        url = '{}'.format(self.url) + ';' +'{}'.format(self.STATUS_COM_010) + ';' +'{}'.format(self.STATUS_COM_073) + ';' +'{}'.format(self.STATUS_COM_075) + ';' +'{}'.format(self.STATUS_COM_079) + ';' +'{}'.format(self.STATUS_COM_080) + ';' +'{}'.format(self.STATUS_COM_110) + ';' +'{}'.format(self.STATUS_COM_120) + ';' +'{}'.format(self.STATUS_COM_130) + ';' +'{}'.format(self.STATUS_COM_141) + ';' +'{}'.format(self.STATUS_COM_142) + ';' +'{}'.format(self.STATUS_COM_144) + ';' +'{}'.format(self.STATUS_COM_148) + ';' +'{}'.format(self.STATUS_COM_151) + ';' +'{}'.format(self.STATUS_COM_152) + ';' +'{}'.format(self.STATUS_COM_155) + ';' +'{}'.format(self.STATUS_COM_160) + ';' +'{}'.format(self.STATUS_COM_192) + ';' +'{}'.format(self.STATUS_COM_193) + ';' +'{}'.format(self.STATUS_SAC_CYCLES_SEC) + ';' +'{}'.format(self.STATUS_SAC_OVERRUNS) + ';' +'{}'.format(self.STATUS_SAC_STATUS) + ';' +'{}'.format(self.STATUS_WS_ACTIVE_SESSIONS) + ';' +'{}'.format(self.STATUS_WS_CLIENT_CONNECTIONS) + ';' +'{}'.format(self.STATUS_WS_HOST_CONNECTIONS) + ';' +'{}'.format(self.STATUS_WS_MAXIMUM_SESSIONS) + ';' +'{}'.format(self.STATUS_WS_SERVICE) + ';' +'{}'.format(self.STATUS_LAST_UPDATE) 

        # requests doc http://docs.python-requests.org/en/v0.10.7/user/quickstart/#custom-headers
        r = requests.get(url=url, headers=header_token, verify=False)

        return r.json(), r.status_code
    
    def check_tags_data(self):

        #Vars
        retrcode = OK

        #Create tuple with json and status code
        historian_tuple = self.get_tags_data()

        #Get data tags from tuple
        historian_data = historian_tuple[0]  


        perfdata_list = ['{}'.format(self.STATUS_SAC_CYCLES_SEC), '{}'.format(self.STATUS_WS_ACTIVE_SESSIONS), 
                        '{}'.format(self.STATUS_SAC_OVERRUNS), '{}'.format(self.STATUS_WS_CLIENT_CONNECTIONS)]
        
        tags_max_value = {'{}'.format(self.STATUS_SAC_OVERRUNS): 10}

        tags_warn_value = {'{}'.format(self.STATUS_SAC_OVERRUNS): 5}

        msgdata = ''
        msgerror = ''
        retrperfdata = ''
        retrmsg = ''

        #Validate Data
        for i, val in enumerate(historian_data['Data']):
            retrcode = OK
            retrcodetag = OK
            ErrorCode = val.get('ErrorCode', 1)
            TagName = val.get('TagName', '')
            Samples = val.get('Samples', [])
            SamplesN = 0
            
            msgdata += 'Tag: {}, ErrorCode: {} \n'.format(val.get('TagName'), ErrorCode)

            #Validate ErrorCode (0 = Ok)
            if ErrorCode != 0:
                retrcode = CRITICAL
                retrcodetag = CRITICAL

            #Validate Samples
            for x, vsample in enumerate(Samples):
                
                #Def Vars
                SamplesN += 1
                Quality = vsample.get('Quality', 0)
                TimeStamp = vsample.get('TimeStamp', '')
                Value = vsample.get('Value')                
                actual_time = arrow.get()

                msgdata += 'SampleN: {}, Quality: {}, TimeStamp: {}, Value: {} \n'.format(SamplesN, Quality, TimeStamp, Value)
                
                if TagName in perfdata_list:
                    retrperfdata += '{}={};{};{};; '.format(TagName, Value, tags_warn_value.get(TagName, ''), tags_max_value.get(TagName, ''))
                
                #Validate Quality (0 – Bad,  1 – Uncertain, 2 – NA , 3 – Good)
                if Quality != 3:
                    retrcode = CRITICAL
                    retrcodetag = CRITICAL

                #Validate Value (0 = Ok)    
                if Value != '0':
                    retrcode = CRITICAL
                    retrcodetag = CRITICAL

                #STATUS_SAC_STATUS / STATUS_WS_SERVICE - ( 0=STOP | 1=RUN )
                if TagName in [self.STATUS_SAC_STATUS , self.STATUS_WS_SERVICE] and Value != '1':
                    retrcode = CRITICAL
                    retrcodetag = CRITICAL

                #STATUS_LAST_UPDATE (LAST UPDATE > ACTUAL TIME -1 MINUTE)
                if TagName in [self.STATUS_LAST_UPDATE] and Value !='0.0000':      
                    last_update_time = arrow.get(Value , 'DD/MM/YYYY HH:mm:ss')
                    actual = actual_time.shift(hours=-3, minutes=-1)
                    if last_update_time > actual:
                        retrcode = OK
                        retrcodetag = OK  

                #STATUS_SAC_OVERRUNS (Value < 10)          
                if TagName in [self.STATUS_SAC_OVERRUNS]:              
                    if Value < '10':                        
                        retrcode = OK
                        retrcodetag = OK             
                #STATUS_SAC_CYCLES_SEC (Values = 20)
                if TagName in [self.STATUS_SAC_CYCLES_SEC]:
                    if Value == '20':
                        retrcode = OK
                        retrcodetag = OK
                #STATUS_SAC_STATUS and STATUS_WS_SERVICE (Values = 1)
                if TagName in [self.STATUS_SAC_STATUS , self.STATUS_WS_SERVICE]:
                    if Value == '1':
                        retrcode = OK
                        retrcodetag = OK  
            #Only info
            if TagName in [self.STATUS_WS_ACTIVE_SESSIONS , self.STATUS_WS_CLIENT_CONNECTIONS , self.STATUS_WS_HOST_CONNECTIONS , self.STATUS_WS_MAXIMUM_SESSIONS]:
                retrcode = OK
                retrcodetag = OK
            
            if retrcodetag != 0:
                msgerror += 'ERROR: Tagname: {} \n'.format(TagName)

        msgerror += msgdata
 
        return retrcode, msgerror
